# coding: utf-8
import pymel.core as pm
class GUI(object):
	windowname = "AimRigGenerator"
	def __init__(self):
		pass
		
	def buildGUI(self,buildRigFunc=None,setIKFunc=None,closeFunc=None,ver=None):
		_windowname = self.__class__.windowname + ver
		
		#Delete exist window
		if pm.window(_windowname,q=1,ex=1):
			return False
		
		#Annotaion texts
		ann_RigScl = "Set a size of created rigs."
		ann_Length = "Set a distance between tip and target nodes."
		ann_Axis = "Select a axis vector of joint."
		ann_Offset = "AimRigControllers Offset.\nThis values based of worldspace."
		
		#Build main
		with pm.window(_windowname,s=0,mnb=0,mxb=0) as win:
			pm.scriptJob(uid=[win,closeFunc])
			with pm.formLayout() as f:
				#Form attribute value
				formMargin = 10
				margin = 5
				controlHeight = 25
				labelWidth = 80
				#Label Rig Scale
				pm.text("label_rigScl",l="Rig Scale :",w=labelWidth,h=controlHeight,ann=ann_RigScl)
				f.attachForm("label_rigScl","left",formMargin)
				f.attachForm("label_rigScl","top",formMargin)
				#Rig Scale input field
				self.rigScl = pm.floatField(cc=buildRigFunc,pre=1,h=controlHeight,ann=ann_RigScl)
				f.attachForm(self.rigScl,"top",formMargin)
				f.attachForm(self.rigScl,"right",formMargin)
				f.attachControl(self.rigScl,"left",margin,"label_rigScl")
				#Label Length
				pm.text("label_length",l="Length :",w=labelWidth,h=controlHeight,ann=ann_Length)
				f.attachForm("label_length","left",formMargin)
				f.attachControl("label_length","top",margin,"label_rigScl")
				#Length input field
				self.lengthField = pm.floatField(cc=buildRigFunc,h=controlHeight,ann=ann_Length)
				f.attachForm(self.lengthField,"right",formMargin)
				f.attachControl(self.lengthField,"top",margin,"label_rigScl")
				f.attachControl(self.lengthField,"left",margin,"label_length")
				#Label Axis
				pm.text("label_axis",l="Axis :",w=labelWidth,h=controlHeight,ann=ann_Axis)
				f.attachForm("label_axis","left",formMargin)
				f.attachControl("label_axis","top",margin,"label_length")
				#Axis radioButtons
				with pm.rowLayout(nc=3) as axisButtons:
					buttonWidth = 30
					self.axisSelector = pm.radioCollection()
					pm.radioButton("x",cc=buildRigFunc,l="X",w=buttonWidth,h=controlHeight,ann=ann_Axis)
					pm.radioButton("y",cc=buildRigFunc,l="Y",w=buttonWidth,h=controlHeight,ann=ann_Axis)
					pm.radioButton("z",cc=buildRigFunc,l="Z",w=buttonWidth,h=controlHeight,ann=ann_Axis,sl=1)
				f.attachControl(axisButtons,"left",margin,"label_axis")
				f.attachControl(axisButtons,"top",margin,"label_length")
				f.attachForm(axisButtons,"right",formMargin)
				#Separator top
				pm.separator("line_offsetTop",h=margin,st="in")
				f.attachForm("line_offsetTop","left",margin)
				f.attachForm("line_offsetTop","right",margin)
				f.attachControl("line_offsetTop","top",margin,"label_axis")
				#Label OffsetX
				pm.text("label_offsetX",l="OffsetX :",w=labelWidth,h=controlHeight,ann=ann_Offset)
				f.attachForm("label_offsetX","left",formMargin)
				f.attachControl("label_offsetX","top",margin,"line_offsetTop")
				#Label OffsetY
				pm.text("label_offsetY",l="OffsetY :",w=labelWidth,h=controlHeight,ann=ann_Offset)
				f.attachForm("label_offsetY","left",formMargin)
				f.attachControl("label_offsetY","top",margin,"label_offsetX")
				#Label OffsetZ
				pm.text("label_offsetZ",l="OffsetZ :",w=labelWidth,h=controlHeight,ann=ann_Offset)
				f.attachForm("label_offsetZ","left",formMargin)
				f.attachControl("label_offsetZ","top",margin,"label_offsetY")
				#OffsetFieldX
				self.offsetX = pm.floatField(cc=buildRigFunc,h=controlHeight,ann=ann_Offset)
				f.attachControl(self.offsetX,"top",margin,"line_offsetTop")
				f.attachForm(self.offsetX,"right",formMargin)
				f.attachControl(self.offsetX,"left",margin,"label_offsetY")
				#OffsetFieldY
				self.offsetY = pm.floatField(cc=buildRigFunc,h=controlHeight,ann=ann_Offset)
				f.attachControl(self.offsetY,"top",margin,"label_offsetX")
				f.attachForm(self.offsetY,"right",formMargin)
				f.attachControl(self.offsetY,"left",margin,"label_offsetZ")
				#OffsetFieldZ
				self.offsetZ = pm.floatField(cc=buildRigFunc,h=controlHeight,ann=ann_Offset)
				f.attachControl(self.offsetZ,"top",margin,"label_offsetY")
				f.attachForm(self.offsetZ,"right",formMargin)
				f.attachControl(self.offsetZ,"left",margin,"label_offsetZ")
				#Separator bottom
				pm.separator("line_offsetBottom",h=margin,st="in")
				f.attachForm("line_offsetBottom","left",margin)
				f.attachForm("line_offsetBottom","right",margin)
				f.attachControl("line_offsetBottom","top",margin,"label_offsetZ")
				#SetIK button
				pm.button("button_set",l="Set",h=controlHeight*2,c=setIKFunc)
				f.attachForm("button_set","left",formMargin)
				f.attachControl("button_set","top",margin,"line_offsetBottom")
				f.attachForm("button_set","bottom",formMargin)
				#Cancel button
				pm.button("button_cancel",l="Cancel",h=controlHeight*2,c=lambda *arg:pm.deleteUI(win))
				f.attachControl("button_cancel","top",margin,"line_offsetBottom")
				f.attachForm("button_cancel","right",formMargin)
				f.attachPosition("button_cancel","left",0,50)
				f.attachControl("button_set","right",margin,"button_cancel")
				f.attachForm("button_cancel","bottom",formMargin)
		return True