# coding: utf-8

'''AimRigGenerator
 
How to use : Register to shelf following 2 line as Python.
 
import okAimRigGenerator
okAimRigGenerator.main()
 
Auther: Naoto Okumura, Canplaize Inc.
License: CC BY-SA 4.0
'''


import pymel.core as pm
import AimRigGenerator_GUI

class AimRigGenerator(object):
	ver = "1.2"
	tipShape = {"x":[0, 1, 0,-1, 0,1,0,-1, 0, 0, 0,0,0],
				"y":[1, 0,-1, 0, 0,0,0, 0, 1, 0,-1,0,1],
				"z":[0, 0, 0, 0,-1,0,1, 0, 0,-1, 0,1,0]}
	upvShape = {"x":[0, 1, 1,0, 1,-1,0,-1,-1,0,-1, 1,0],
				"y":[1,-1,-1,1,-1,-1,1,-1,-1,1,-1,-1,1],
				"z":[0, 1,-1,0,-1,-1,0,-1, 1,0, 1, 1,0]}
	
	def __init__(self):
		print "Instantiated AimRigGenerator" , self.__class__.ver
		self.gui = AimRigGenerator_GUI.GUI()
		self.rigroots = []
		
	def valuesInit(self):
		#Reset my class values
		if self.rigroots : pm.delete(self.rigroots)
		self.rigroots = []
		self.bakeNodes = []
		self.constraints = {"Constraint":[] , "TargetNodes":[]}
		self.offset = []
		self.deleteNodes = []
	
	def buildRig(self,first=False):
		self.valuesInit()
		#Get user values
		if first:
			nodeSize = self.getNodeSize(self.selection[0])
			self.gui.rigScl.setValue(nodeSize*0.2)
			self.gui.lengthField.setValue(nodeSize)
		rigScale = self.gui.rigScl.getValue()
		length = self.gui.lengthField.getValue()
		axis = self.gui.axisSelector.getSelect()
		x_offset = self.gui.offsetX.getValue()
		y_offset = self.gui.offsetY.getValue()
		z_offset = self.gui.offsetZ.getValue()
		for node in self.selection:
			#Selected node infomation
			targetName = node.name()
			targetPos = pm.xform(node,q=1,ws=1,t=1)
			targetRot = pm.xform(node,q=1,ws=1,ro=1)
			
			#Create rig main
			locator = lambda part : pm.spaceLocator(n=targetName + part)
			root = locator("_IKroot")
			
			#Guide Locators
			guideStart = locator("_Start")
			guideEnd = locator("_End")
			guideOffset = locator("_Offset")
			self.setRelation([guideOffset,guideEnd,guideStart,root])
			guideEnd.setAttr("t"+axis,length)
			
			#Joint and Ik handle
			jointRoot = pm.joint(p=pm.xform(guideStart,q=1,ws=1,t=1),n=targetName+"_rootJoint")
			jointEnd = pm.joint(p=pm.xform(guideEnd,q=1,ws=0,t=1),n=targetName+"_endJoint",r=1)
			jointHandle = pm.ikHandle(sj=jointRoot,ee=jointEnd,n=targetName+"_IKHandle",sol="ikRPsolver")[0]
			self.setRelation([jointRoot,guideEnd])
			self.setRelation([jointHandle,root])
			handle = self.createShape(self.__class__.tipShape,rigScale,targetName+"_Handle",14)
			handleOffset = locator("_HandleOffset")
			self.setRelation([handleOffset,handle,root])
			
			#Upvector
			upvLocal = locator("_upv")
			upvGlobal = self.createShape(self.__class__.upvShape,rigScale*0.5,targetName+"_Upvector",13)
			self.setRelation([upvLocal,guideStart])
			self.setRelation([upvGlobal,root])
			upvLocal.setAttr("t",[0,0,0])
			upvLocal.setAttr("t"+axis,length*0.5)
			if axis == "z" or axis == "x":
				upvLocal.setAttr("ty",length)
			else:
				upvLocal.setAttr("tz",length)
			
			#Rotation Offset
			rotOffset = locator("_rotOffset")
			rotOffset.setParent(jointRoot)
			
			#Set Constraints
			pm.pointConstraint(node,guideStart)
			nodeOriConst = pm.orientConstraint(node,guideStart)
			guideEndConst = pm.pointConstraint(guideEnd,jointHandle)
			upvLocalConst = pm.pointConstraint(upvLocal,upvGlobal)
			jointHandleConst = pm.pointConstraint(jointHandle,handleOffset)
			guideOffsetConst = pm.pointConstraint(guideOffset,handle)
			pm.orientConstraint(node,rotOffset)
			
			#Offset
			position = pm.xform(guideOffset,q=1,ws=1,t=1)
			position[0] += x_offset
			position[1] += y_offset
			position[2] += z_offset
			pm.xform(guideOffset,t=position,ws=1)
			
			#Add values to list
			self.rigroots.append(root)
			self.bakeNodes.extend([handle,upvGlobal,handleOffset])
			self.deleteNodes.extend([nodeOriConst,guideEndConst,upvLocalConst,jointHandleConst,guideOffsetConst])
			self.constraints["Constraint"].append(pm.pointConstraint) ; self.constraints["TargetNodes"].append([upvGlobal,upvLocal])
			self.constraints["Constraint"].append(pm.pointConstraint) ; self.constraints["TargetNodes"].append([handleOffset,jointHandle])
			self.constraints["Constraint"].append(pm.poleVectorConstraint) ; self.constraints["TargetNodes"].append([upvLocal,jointHandle])
			self.offset.append([rotOffset,node])
			
	def setRelation(self,targetList):
		for node in enumerate(targetList):
			if node[0] < len(targetList)-1:
				node[1].setParent(targetList[node[0]+1])
	
	def getNodeSize(self,node):
		shape = node.getShape()
		start = shape.boundingBox()[0]
		end = shape.boundingBox()[1]
		size = sum([(valueA - valueB)**2 for valueA,valueB in zip(start,end)])**0.5
		return size
	
	def setIK(self):
		inFrame = int(pm.playbackOptions(q=1,min=1))
		outFrame = int(pm.playbackOptions(q=1,max=1))
		pm.bakeResults(self.bakeNodes,at="translate",t=(inFrame,outFrame),sm=1)
		pm.delete(self.deleteNodes)
		for constraint,targetNodes in zip(self.constraints["Constraint"],self.constraints["TargetNodes"]):
			constraint(targetNodes[0],targetNodes[1])
		for node in self.offset:
			pm.delete(node[0].connections(type="orientConstraint"))
			pm.orientConstraint(node[0],node[1])
		self.rigroots = None
		pm.deleteUI(self.gui.__class__.windowname)
		pm.confirmDialog(t="SetIK",m="Processing completed.")
	
	def createShape(self,points,size,name,color):
		drow = []
		for point in zip(points["x"],points["y"],points["z"]):
			drow.append(map(lambda value:value*size , point))
		rig = pm.curve(d=1,p=drow,n=name)
		shape = rig.getShape()
		shape.setAttr("overrideEnabled",1)
		shape.setAttr("overrideColor",color)
		return rig
	
	def checkSelection(self,selectedNodeList):
		notTransformNodes = [node for node in selectedNodeList if type(node) != pm.nodetypes.Transform]
		haveOrientConstraintNodes = [node for node in selectedNodeList if node.connections(type="orientConstraint",d=0)]
		resultNodes = [node for node in selectedNodeList if not ((node in notTransformNodes) or (node in haveOrientConstraintNodes))]
		errorString = "Can not set IK from this script\n"
		if notTransformNodes:
			errorString += "Following nodes is not TransformNode\n"
			for node in notTransformNodes: errorString += " " + node.name()+"\n"
		if haveOrientConstraintNodes:
			errorString += "Following nodes already have OrientConstraint.\n"
			for node in haveOrientConstraintNodes: errorString += " " +node.name()+"\n"
		if haveOrientConstraintNodes or notTransformNodes:
			pm.confirmDialog(title="Warning",m=errorString)
		if not selectedNodeList:
			pm.confirmDialog(title="Warning",m="Target not found...")
		return resultNodes
	
	def main(self):
		self.selection = self.checkSelection(pm.selected())
		if not self.selection : return
		_buildRigFunc = lambda *arg : self.buildRig(first=False)
		_setIKFunc = lambda *arg : self.setIK()
		_closeFunc = lambda *arg : self.valuesInit()
		if self.gui.buildGUI(_buildRigFunc,_setIKFunc,_closeFunc,self.__class__.ver):
			self.buildRig(first=True)

def main():
	func = AimRigGenerator()
	func.main()
if __name__ == "__main__" : main()